<?php
namespace FiloBlu\Storelocator\Model;

use Magento\Framework\Exception\LocalizedException as CoreException;

class Tags extends \Magento\Framework\Model\AbstractModel
{

    protected $tagsCollectionFactory;

    /**
     * Store manager
     *
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    public function __construct(
        \FiloBlu\Storelocator\Model\ResourceModel\Tags\CollectionFactory $tagsCollectionFactory,
        \Magento\Store\Model\StoreManagerInterface $_storeManager,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    )
    {
        $this->tagsCollectionFactory = $tagsCollectionFactory;
        $this->_storeManager = $_storeManager;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init('FiloBlu\Storelocator\Model\ResourceModel\Tags');
    }


    public function getAvailableStatuses()
    {
        return array(
            0 => "Disabled",
            1 => "Enabled"
        );
    }

    public function getAllTags()
    {
        $collection = $this->tagsCollectionFactory->create();
        $collection->addFieldToFilter('enabled', ["eq" => 1]);

        return $collection;
    }

    public function getAllAvailableTags()
    {
        $collection = $this->getAllTags();

        $collection->getSelect()->join(
            array('related' => $collection->getTable('filoblu_storelocator_store_tag')),
            'related.tag_id=main_table.tag_id',
            array('main_table.tag_id', 'main_table.image',  'main_table.description')
        );

        $collection->getSelect()->group(array('tag_id','image','description'));

        foreach ($collection as $tag) {
            if ($tag->getImage()) {
                $tag->setImageUrl($this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . "filoblu/storelocator/stores/image/" . $tag->getImage());
            }
        }

        return $collection;
    }

}